#include "bjlConnection.h"

CBJLConnection::CBJLConnection(int port)
{
// write default values
	if (port)
		control.port=port;
	else
		control.port=0x378;

	control.checkSum=false;
	control.packets=false;
	control.timeout=10000000;

	handler=-1;
}

void CBJLConnection::Open()
{
	handler=open("/dev/misc/bjlServer", O_RDWR| O_NONBLOCK);
	ioctl(handler, CONFIGURE, &control, 0);	
}

void CBJLConnection::Close()
{
	close(handler);
	handler=-1;
}

bool CBJLConnection::IsReady()
{
	return (handler!=-1);
}

void CBJLConnection::Configure(io_control_data_t *io)
{
	control=*io;
	ioctl(handler, CONFIGURE, &control, 0);
}

void CBJLConnection::SendLong(int longword) const
{
	int data=B_BENDIAN_TO_HOST_INT32(longword);
	write(handler, &data, 4);
}

void CBJLConnection::SendBuffer(void *buffer, int size) const
{
	write(handler, buffer, size);
}

bool CBJLConnection::ReceiveBool() const
{
	ioctl(handler, READ_STATUS, &control, 0);

	return control.ok;
}

unsigned char CBJLConnection::ReceiveByte() const
{
	unsigned char data;
	read(handler, &data, 1);

	return data;
}

unsigned short CBJLConnection::ReceiveWord() const
{
	unsigned short data;
	read(handler, &data, 2);

	return data;
}

unsigned long CBJLConnection::ReceiveLong() const
{
	unsigned long data;
	read(handler, &data, 4);

	return data;
}

void CBJLConnection::GetErrors(float *packets, float *fails)
{
	ioctl(handler, GET_ERRORS, &control, 0);
	*packets=control.numPackets;
	*fails=control.numFails;
}

void CBJLConnection::EnableCheckSum(bool enable)
{
	control.checkSum=enable;
	ioctl(handler, SET_CHECK_SUM, &control, 0);
}

void CBJLConnection::EnablePackets(bool enable)
{
	control.packets=enable;
	ioctl(handler, SET_PACKETS, &control, 0);
}

