
/*
** File:	bjlclass.cpp
** Date:	03.07.02
** Info:	BJL Class library to open, send, configure and close the BJL driver
** Ver:	0.005
**
*/


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// I N C L U D E S
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
#include "bjlclass.h"



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// *** Constructor: Best regards to Patrick G. and his mentor Mental Theo ... :-)
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
Bjl::Bjl ()
{
	#ifdef DEBUG
	cout << "Initalizing BJL class ..." << "\n";
	#endif
	
	iocd.Port		= 0x378;
	iocd.buffer	= 0;
	iocd.bufferSize	= 0;
	iocd.addr		= 0x4000;

	fd			= -1;
}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// * Destructor:
// * Nothing to destory yet.
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
Bjl::~Bjl()
{}



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// * Method: 	Open
// * Info:		Get the BJL Driver handle
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

void Bjl::Open()
{
	#ifdef DEBUG
	cout << "Opening BJL driver ..." << "\n";
	#endif
	
	fd = open("/dev/misc/bjl", O_RDWR| O_NONBLOCK);	
	
}


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// * Method: 	Close
// * Info:		Close the BJL Driver handle
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void Bjl::Close()
{
	#ifdef DEBUG
	cout << "Closing BJL driver ..."<<"\n";
	#endif
	
	close (fd);
}



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// * Method: 	isReady
// * Info:		Close the BJL Driver handle
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
bool Bjl::isReady()
{
	return ( (fd == -1) ? false: true);
}




// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// * Method: 	Configure
// * Info:		Set the BJL driver working parameters.
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void Bjl::Configure (IOCtlData *iods)
{
	iocd.Port 		= iods->Port;
	iocd.buffer	= iods->buffer;
	iocd.bufferSize	= iods->bufferSize;
	iocd.addr		= iods->addr;
	
	#ifdef DEBUG
	cout << "New configuration set ..." << "\n";
	#endif	
}




// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// * Method: 	Send
// * Info:		Call the drivers with BJL_SEND method passing the
//			IOCtlData.
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void Bjl::Send()
{
	if ((iocd.buffer == 0) || (iocd.bufferSize == 0))
		cout<<"Nothing to send !!!"<<"\n";
	else {
		ioctl(fd, BJL_SEND, &iocd, 0);
		}		
}
