; send/receive data to/from pc, without timeout
;
; Protocol to send a byte from pc to jaguar
;
;+--------------------------------------------------------------------+
;|          PC side                |          Jaguar side             |
;+---------------------------------+----------------------------------+
;| Wait for BUSY to be low         | Set BUSY low                     | 
;| Send LSB                        | Wait for /STROBE to be low       |
;| Set /STROBE to low              | Read data from joyport           |
;| Wait for BUSY high              | Set BUSY high                    |
;| Set /STROBE to high             | Wait for /STROBE to be high      |
;+--------------------------------------------------------------------+
;
; Protocol to send a byte jaguar to pc (bit by bit)
;
;+--------------------------------------------------------------------+
;|        Jaguar side              |            PC side               |
;+---------------------------------+----------------------------------+
;| Set BUSY low                    | Wait for BUSY to be low          | 
;| Wait for /STROBE to be low      | Set /STROBE to low               |
;|                                 | delay                            |
;| Set BUSY equal high bit         | Read BUSY bit                    |
;|                                 | delay                            |
;| Wait for /STROBE to be high     | Set /STROBE to high              |
;|                                 | delay                            |
;| Loop for next bit               | Loop for next bit                |
;+--------------------------------------------------------------------+
;
; routines are named from jaguar side, this means that SendByte, sends
; a byte from Jaguar to PC

	.68000					; don't forget this

TIMEOUT	equ 30000

SendByte
	movem.l d0/d1/d7/a0,-(sp)
	lea $f14000,a0			; Joystick

	move.w #$8170,(a0)		; busy low = 0

	move.w #8-1,d7
.bit_loop
.waitStrobeLow
	move.l (a0),d1
	btst #3,d1
	bne.s .waitStrobeLow

	add.b d0,d0				; shift left and carry=bit to send
	bcc.s .send_zero
	move.w #$81f0,(a0)		; busy high = 1
	bra.s .skip
.send_zero
	move.w #$8170,(a0)		; busy low = 0
.skip

.waitStrobeHigh
	move.l (a0),d1
	btst #3,d1
	beq.s .waitStrobeHigh
	dbf d7,.bit_loop		; send next bit

	movem.l (sp)+,d0/d1/d7/a0
	rts

; get a byte form joyport and return it into d0
ReceiveByte
	movem.l d1-2/a0,-(sp)
	lea $f14000,a0			; Joystick

	move.w #TIMEOUT,d2		; time out
	move.w #$8170,(a0)		; busy low = 0

.waitStrobeLow
	subq.w #1,d2
	beq.s .timeOut
	move.w 2(a0),d0			; read buttons register
	btst #3,d0
	bne.s .waitStrobeLow

	move.w #$100,(a0)		; switch to input & set BUSY high
	move.w (a0),d1			; get data bits 7..4 & 2..0

;31 30 29 28 27 26 25 24|23 22 21 20 19 18 17 16|15 14 13 12 11 10 9 8|7 6 5 4 3 2 1 0
; 7  6  5  4  x  x  x  x B  2   1  0  x  x  x  x  x  x  x  x  x  x x x x x x x S 3 x x
	lsr.w #4,d1			;
	and.w #$0f07,d1		; mask out bits

	and.w #$4,d0		; get bit 3
	add.w d0,d0
	or.b d1,d0			; d0=low nibble

	lsr.w #4,d1			; bits 11..8(7..4) => 7..4
	or.b d1,d0

.waitStrobeHigh
	subq.w #1,d2
	beq.s .timeOut
	move.w 2(a0),d1			; read joyport data
	btst #3,d1
	beq.s .waitStrobeHigh

	move.b #0,ccr			; carry clear ok
	movem.l (sp)+,d1-2/a0
	rts

.timeOut
	move.w #$81f0,(a0)		; busy high = 1
	move.b #1,ccr			; carry set => timeout!
	movem.l (sp)+,d1-2/a0
	rts

; block routines
; d0.w size in bytes
; a0 address
SendBuffer
	move.w d0,d1
.loop
	move.b (a0)+,d0
	bsr SendByte
	dbf d1,.loop
	rts

