// Packet Loader
// 10/05/2005	(c) Roberto Nadal Martinez (swapd0@yahoo.es)
// This is a packet loader for BJL.
// Upload address is before user ram ($4000), this means that the loader
// must be as small as you can, or you'll smash original BJL loader.
// Includes remote debugger routines, to put the jaguar into debug mode
// just put a trap #0 into your code, and then executes jdb, pc side of the
// debugger, or keep #0 pressed in keypad 1 at the end of uploading

SCREEN_WIDTH	equ 32
SCREEN_HEIGHT	equ 3
textScreen		equ $300

	.68000
	org $3000
	start main

	include "Jaguar.inc"

main
	lea $1000.w,sp
	bsr Init				// init jaguar
	bsr InitDebugger		// install remote debugger

	move.w #SCREEN_WIDTH,d0
	move.w #SCREEN_HEIGHT,d1
	bsr SetScreenSize		; init print routines

	move.l #$807f80ff,CLUT
	move.w #1,VI

	lea gpuCode(pc),a0
	movea.l #G_RAM,a1
	move.w #gpuCodeEnd-gpuCode,d0
	bsr LoadGPU
	bsr GoGPU
	bsr WaitGPU

	movea.l 0.w,a0					// read address at 0.w
	move.l a0,startAddress			// start address
	move.l #0,0.w					//
	move.l #4,4.w					// write stop object
	move.l #0,OLP					// stop OP
	move.w #$2700,sr
	move.w #$7fff,VI				// no Vertical Interrupt
	move.w #$1f00,INT1				// clear pending IRQs

// read joypad
	move.w #$81fb,JOYSTICK
	move.w JOYSTICK,d0
	btst #8,d0					// press 0 to debug
	bne.s .noDebug
	trap #0
.noDebug
	jmp (a0)

Init
	move.w #$2700,sr			// disable all interrups
	lea _BASE,a0
	move.l #$070007,d0			// big endian
	move.l d0,G_END-_BASE(a0)
	move.l d0,D_END
	moveq #0,d0
	move.l d0,G_CTRL-_BASE(a0)	// stop gpu
	move.l d0,D_CTRL			// stop dsp
	move.l d0,G_FLAGS-_BASE(a0)	// disable GPU-IRQs
								// disable DSP-IRQs
	move.l #%111111<<8,J_INT    // clear and disable IRQs

	move.l d0,0.w
	moveq #4,d0
	move.l d0,4.w
	moveq #0,d0
	move.l d0,OLP-_BASE(a0)		// set OP to STOP-OBJECT
	move.w d0,OBF-_BASE(a0)		// clear OP-Flag
	move.l d0,BORD1-_BASE(a0)	// border black
	move.w d0,BG-_BASE(a0)		// background black
	move.l d0,PIT0-_BASE(a0)	// stop PIT
	move.l d0,JPIT1				// stop JPIT1
	move.l d0,JPIT3				// stop JPIT2
	move.w #$1F01,INT1-_BASE(a0)	// clear pending irqs & enable vbl
	move.w #$7fff,VI-_BASE(a0)	// no vbi
	lea vbl_irq(pc),a0
	move.l a0,LEVEL0.w

// video init
	move.w CONFIG,d0               // Also is joystick register
	andi.w #VIDTYPE,d0             // 0 = PAL, 1 = NTSC
	beq.s is_pal

	move.w #NTSC_HMID,d2
	move.w #NTSC_WIDTH,d0
	move.w #NTSC_VMID,d6
	move.w #NTSC_HEIGHT,d4
	bra.s calc_values

is_pal
	move.w #PAL_HMID,d2
	move.w #PAL_WIDTH,d0
	move.w #PAL_VMID,d6
	move.w #PAL_HEIGHT,d4
calc_values
	move.w d0,d1
	asr.w #1,d1                   // Width/2
	sub.w d1,d2                   // Mid - Width/2
	add.w #4,d2                   // (Mid - Width/2)+4
	sub.w #1,d1                   // Width/2 - 1
	ori.w #$400,d1                // (Width/2 - 1)|$400
	move.w d1,HDE
	move.w d2,HDB1
	move.w d2,HDB2

	move.w d6,d5
	sub.w d4,d5

	move.w d5,VDB
	move.w #$FFFF,VDE

	move.l #0,BORD1				// Black border
	move.w #0,BG				// Init line buffer to black

	move.w #$6c1,VMODE			// CRY-mode (non-overscanned)

	move.w #$2000,sr			// enable all interrups
	rts

vbl_irq
	movem.l d0-7/a0-6,-(sp)

	move.l $3ffc.w,d0
	lea bytesText(pc),a0
	bsr HexToAscii

	moveq #0,d0
	move.w d0,d1
	bsr SetCursor

	lea loaderText(pc),a0
	lea textScreen.w,a1
	bsr Print

	bsr BuildList			// guess...
	
// swap lists
	move.l currentList(pc),d0
	move.l workingList(pc),d1
	move.l d1,currentList
	move.l d0,workingList

	add.l #16,d1			// skip first stop object & padding
	swap d1					// don't forget this!!!
	move.l d1,OLP

	move.w sr,d0
	andi.w #$400,d0
	bne.s BailOut

	move.w #$101,INT1		// clear vbl interrupt and enable it
	move.w #0,INT2			// restore interrupts ***

	movem.l (sp)+,d0-7/a0-6
	rte

BailOut
	move.w #$1816,$f00000
	move.l #$e00004,-(sp)
	rts

bGreater	equ 1 << 14
bLess		equ 2 << 14

// hard coded object list builder
BuildList
	movea.l workingList(pc),a0

	movea.l a0,a1			// save beginning of the list
	bsr.s AddStop			// first a stop object
	lea 8(a0),a0			// skip one phrase (for alignment)

	move.w #596,d0			// video end
	move.w #bLess,d1		// condition
	move.l a1,d2			// link address (first stop object)
	bsr.s AddBranch

	move.w #36,d0			// video start ( first visible scan line=36 )
	move.w #bGreater,d1		// condition
	move.l a1,d2			// link address (first stop object)
	bsr.s AddBranch

	move.w #(360-32*8)/2,d0	// screen center
	move.w #128*2,d1
	move.l #textScreen,d2
//	lea textScreen(pc),a1
//	move.l a1,d2
	bsr.s AddTextScreen

	bsr.s AddStop
	rts

// Add a Branch object to the list
// parameters:
// a0 = memory address to write the branch object
// d0.w = line number
// d1.w = condition (in high bits!) use bGreater or bLess equs
// d2.l = link address
// output:
// the object is writen and a0 points to next object in the array
// d0-3 registers are smashed
AddBranch
	lsr.l #3,d2			// address in phrases
	move.l d2,d3		// save address
	lsr.l #8,d2			// address >> 8 bits
	move.l d2,(a0)+		// write high longword
	move.l d3,d2		// get link address
	moveq #24,d3		// bit positon of link field
	lsl.l d3,d2			// shift into link field
	or.w #3,d2			// branch type
	lsl.w #3,d0			// shift y position
	or.w d0,d2			// or in
	or.w d1,d2			// or condition
	move.l d2,(a0)+
	rts

// Add a Stop object to the list
AddStop
	moveq #0,d0
	move.l d0,(a0)+
	moveq #4,d0
	move.l d0,(a0)+
	rts

// Add a Bitmap (32x3x1 characters) object to the list
// parameters:
// a0 = memory address to write the bitmap object
// d0.w = x position
// d1.w = y position
// d2.l = bitmap data
// output:
// the object is writen and a0 points to next object in the array
AddTextScreen
// first phrase
	movem.l d0-4/a1,-(sp)
	moveq #SCREEN_HEIGHT*8,d4			// screen height
	move.w #39,d3			// first line
	cmp.w d3,d1
	bgt.s .no_clip
	sub.w d1,d3				// half lines to skip
	asr.w #1,d3				// lines to skip
	sub.w d3,d4				// reduce height
	bgt.s .no_neg			// don't wanna neg numbers
	movem.l (sp)+,d0-4/a1	// exit if height < 0
	rts
.no_neg
	ext.l d3				// skip data
	lsl.l #6,d3				// *64
	add.l d3,d2
.no_clip
	lsl.l #8,d2				// shift to position
	and.l #$fffff800,d2		// remove bits

	lea 16(a0),a1			// link address is two phrases ahead
	move.l a1,d3
	lsr.l #8,d3				//
	lsr.l #3,d3				// take top 11 bits
	or.l d3,d2				// or data
	move.l d2,(a0)+			// write high longword

	move.l a1,d2			// link address
	moveq #21,d3
	lsl.l d3,d2
	and.l #$ff000000,d2		// low byte at high bits

	swap d4
	lsr.l #2,d4
	or.l d4,d2				// fixed height
	and.w #$7ff,d1			// remove unused bits !!!!
	lsl.w #3,d1				// y position shifted 3 bits left
	or.w d1,d2
	move.l d2,(a0)+			// write low longword

// second phrase
	move.l #(0 << 15),d2	// transparent flag
	move.l d2,(a0)+

// deepth | pitch | dwidth | iwidth
	move.l #(0 <<12) | (1 << 15) | (4 << 18) | (4 << 28),d2
	and.w #$fff,d0			// remove unused bits !!!!
	or.w d0,d2				// or x position
	move.l d2,(a0)+

	movem.l (sp)+,d0-4/a1
	rts

	include "debugger.js"
	include "gpu.js"
	include "print.js"

// Data section
	even
gpuCode
	incbin "comm.bin"
gpuCodeEnd
	even

loaderText
//		  00000000001111111111222222222233
//		  01234567890123456789012345678901
	dc.b "-------> PACKET  LOADER <-------"
	dc.b "        (C) 2005 SWAP D0        "
	dc.b "     BYTES TO GO $"
bytesText
	dc.b "xxxxxxxx      ",0

	align long
// this is the Object processor list
currentList
	dc.l opList1
workingList
	dc.l opList2

	align dphrase
opList1
	ds.l 16

	align dphrase
opList2
	ds.l 16
